@@ -4,6 +4,8 @@
 			-------------------
 begin                : Wed June 7 2004
 copyright            : (C) 2004 by John Ratke
+                     : (C) 2002-2004 Nadeem Hasan <nhasan@kde.org>
+                     : (C) 2002-2004 Ian Geiser <geiseri@kde.org>
 email                : jratke@comcast.net
 ***************************************************************************/
 
@@ -16,24 +18,24 @@ email                : jratke@comcast.net
 *                                                                         *
 ***************************************************************************/
 
-#include "config.h"
-
-#include <qdatetime.h>
-#include <kdebug.h>
-#include <math.h>
-
 #include "metar_parser.h"
 #include "stationdatabase.h"
 #include "sun.h"
 
+#include <QtCore/QDateTime>
+
+#include <kdebug.h>
+
+#include <math.h>
+
 // Temperature conversion macros
 #define TEMP_C_TO_F(x) ( ((x) * 9/5) + 32 )
 #define TEMP_F_TO_C(x) ( ((x) - 32) * 5/9 )
 
 MetarParser::MetarParser(StationDatabase *stationDB,
 			 KLocale::MeasureSystem units,
-			 QDate date,
-			 QTime time,
+			 const QDate &date,
+			 const QTime &time,
 			 unsigned int localUTCOffset) : 
 	m_stationDb(stationDB), m_units(units), m_date(date), m_time(time), m_localUTCOffset(localUTCOffset)
 {
@@ -52,7 +54,7 @@ MetarParser::MetarParser(StationDatabase *stationDB,
 void MetarParser::reset()
 {
 	// Initialize the WeatherInfo structure
-	weatherInfo.theWeather = QString::null;
+	weatherInfo.theWeather.clear();
 	weatherInfo.clouds = 0;
 	weatherInfo.windMPH = 0;
 	weatherInfo.tempC = 0;
@@ -62,15 +64,15 @@ void MetarParser::reset()
 	weatherInfo.qsCurrentList.clear();
 	weatherInfo.qsDate = m_date;
 	weatherInfo.qsTime = m_time;
-	weatherInfo.qsPressure = QString::null;
-	weatherInfo.qsTemperature = QString::null;
-	weatherInfo.qsDewPoint = QString::null;
-	weatherInfo.qsRelHumidity = QString::null;
-	weatherInfo.qsVisibility = QString::null;
-	weatherInfo.qsWindSpeed = QString::null;
-	weatherInfo.qsWindChill = QString::null;
-	weatherInfo.qsHeatIndex = QString::null;
-	weatherInfo.qsWindDirection = QString::null;
+	weatherInfo.qsPressure.clear();
+	weatherInfo.qsTemperature.clear();
+	weatherInfo.qsDewPoint.clear();
+	weatherInfo.qsRelHumidity.clear();
+	weatherInfo.qsVisibility.clear();
+	weatherInfo.qsWindSpeed.clear();
+	weatherInfo.qsWindChill.clear();
+	weatherInfo.qsHeatIndex.clear();
+	weatherInfo.qsWindDirection.clear();
 	weatherInfo.stationNeedsMaintenance = false;
 }
 
@@ -80,10 +82,10 @@ struct WeatherInfo MetarParser::processData(const QString &stationID, const QStr
 	
 	weatherInfo.reportLocation = stationID;
 	
-	kdDebug(12006) << "Processing data: " << metar << endl;
+	kDebug(12006) << "Processing data: " << metar;
 
 	// Split at whitespace into tokens
-	QStringList dataList = QStringList::split(QRegExp("\\s+"), metar);
+	QStringList dataList = metar.split( QRegExp("\\s+"));
 	bool found = false;
 	bool beforeRemark = true;
 
@@ -91,13 +93,13 @@ struct WeatherInfo MetarParser::processData(const QString &stationID, const QStr
 	     it != dataList.end(); ++it)
 	{
 		// Don't try to parse the ICAO location code
-		if ((!found) && (*it == weatherInfo.reportLocation.upper().stripWhiteSpace()))
+		if ((!found) && (*it == weatherInfo.reportLocation.toUpper().trimmed()))
 		{
 			found = true;
 			continue;
 		}
 
-		kdDebug(12006) << "Processing Token: " << *it << endl;
+		kDebug(12006) << "Processing Token: " << *it;
 		
 		if (*it == "RMK")
 		{
@@ -133,12 +135,9 @@ struct WeatherInfo MetarParser::processData(const QString &stationID, const QStr
 		}
 	}
 
-	if (!weatherInfo.stationNeedsMaintenance)
-	{
-		calcTemperatureVariables();
-		calcWindChill();
-		calcCurrentIcon();
-	}
+	calcTemperatureVariables();
+	calcWindChill();
+	calcCurrentIcon();
 	
 	return weatherInfo;
 }
@@ -148,7 +147,7 @@ bool MetarParser::parseCover(const QString &s)
 {
 	if (CoverRegExp.search(s) > -1)
 	{
-		kdDebug(12006) << "Cover: " << CoverRegExp.capturedTexts().join("-")
+		kDebug(12006) << "Cover: " << CoverRegExp.capturedTexts().join("-")
 				<< endl;
 
 		QString sCode = CoverRegExp.cap(1);
@@ -161,32 +160,32 @@ bool MetarParser::parseCover(const QString &s)
 		{
 			height = height * 0.3048;
 			// using plural i18n form for proper translations
-			sClouds = i18n("1 meter", "%n meters", (int)height);
+			sClouds = i18np("1 meter", "%1 meters", (int)height);
 		}
 		else
 		{
 			// using plural i18n form for proper translations
-			sClouds = i18n("1 foot", "%n feet", (int)height);
+			sClouds = i18np("1 foot", "%1 feet", (int)height);
 		}
 
 		if (sCode == "FEW")
 		{
-			skycondition = i18n( "Few clouds at %1" ).arg(sClouds);
+			skycondition = i18n( "Few clouds at %1" , sClouds);
 			weatherInfo.clouds += 2;
 		}
 		else if (sCode == "SCT")
 		{
-			skycondition = i18n( "Scattered clouds at %1" ).arg(sClouds);
+			skycondition = i18n( "Scattered clouds at %1" , sClouds);
 			weatherInfo.clouds += 4;
 		}
 		else if (sCode == "BKN")
 		{
-			skycondition = i18n( "Broken clouds at %1" ).arg(sClouds);
+			skycondition = i18n( "Broken clouds at %1" , sClouds);
 			weatherInfo.clouds += 8;
 		}
 		else if (sCode == "OVC")
 		{
-			skycondition = i18n( "Overcast clouds at %1" ).arg(sClouds);
+			skycondition = i18n( "Overcast clouds at %1" , sClouds);
 			weatherInfo.clouds += 64;
 		}
 		else if ((sCode == "CLR") || (sCode == "SKC") || (sCode == "CAVOK"))
@@ -195,7 +194,7 @@ bool MetarParser::parseCover(const QString &s)
 			weatherInfo.clouds = 0;
 		}
 
-		kdDebug(12006) << "*** Clouds: " << weatherInfo.clouds << endl;
+		kDebug(12006) << "*** Clouds: " << weatherInfo.clouds;
 		weatherInfo.qsCoverList << skycondition;
 
 		return true;
@@ -210,97 +209,97 @@ bool MetarParser::parseCurrent(const QString &s)
 	{
 		QString sIntensity = CurrentRegExp.cap(1);
 		QString sCode = CurrentRegExp.cap(2);
-		QString currentWeather = "";
+		QString intensity, descriptor, phenomena, currentWeather;
 
-		kdDebug(12006) << "Current: " << CurrentRegExp.capturedTexts().join("-") << endl;
+		kDebug(12006) << "Current: " << CurrentRegExp.capturedTexts().join("-");
 
 		// Decode the intensity
 		if (sIntensity == "+")
 		{
-			currentWeather = i18n("Heavy");
+			intensity = i18n("Heavy");
 			weatherInfo.heavy = true;
 		}
 		else if (sIntensity == "-")
 		{
-			currentWeather = i18n("Light");
+			intensity = i18n("Light");
 			weatherInfo.heavy = false;
 		}
 
 		// Decode the descriptor
 		if (sCode.contains("MI"))
-			currentWeather += i18n(" Shallow");
+			descriptor = i18n("Shallow");
 		else if (sCode.contains("PR"))
-			currentWeather += i18n(" Partial");
+			descriptor = i18n("Partial");
 		else if (sCode.contains("BC"))
-			currentWeather += i18n(" Patches");
+			descriptor = i18n("Patches");
 		else if (sCode.contains("DR"))
-			currentWeather += i18n(" Low Drifting");
+			descriptor = i18n("Low Drifting");
 		else if (sCode.contains("BL"))
-			currentWeather += i18n(" Blowing");
+			descriptor = i18n("Blowing");
 		else if (sCode.contains("SH"))
 		{
-			currentWeather += i18n(" Showers");
+			descriptor = i18n("Showers");
 			weatherInfo.theWeather = "shower";
 		}
 		else if (sCode.contains("TS"))
 		{
-			currentWeather += i18n(" Thunder Storm");
+			descriptor = i18n("Thunder Storm");
 			weatherInfo.theWeather = "tstorm";
 		}
 		else if (sCode.contains("FZ"))
 		{
-			currentWeather += i18n(" Freezing");
+			descriptor = i18n("Freezing");
 		}
 
 		// Decode weather phenomena
 		if (sCode.contains("DZ"))
 		{
-			currentWeather += i18n(" Drizzle");
+			phenomena = i18n("Drizzle");
 			weatherInfo.theWeather = "light_rain";
 		}
 		else if (sCode.contains("RA"))
 		{
-			currentWeather += i18n(" Rain");
+			phenomena = i18n("Rain");
 			weatherInfo.theWeather = "shower";
 		}
 		else if (sCode.contains("SN"))
 		{
-			currentWeather += i18n(" Snow");
+			phenomena = i18n("Snow");
 			weatherInfo.theWeather = "snow";
 		}
 		else if (sCode.contains("SG"))
 		{
-			currentWeather += i18n(" Snow Grains");
+			phenomena = i18n("Snow Grains");
 			weatherInfo.theWeather = "snow4";
 		}
 		else if (sCode.contains("IC"))
 		{
-			currentWeather += i18n(" Ice Crystals");
+			phenomena = i18n("Ice Crystals");
 			weatherInfo.theWeather = "hail";
 		}
 		else if (sCode.contains("PE"))
 		{
-			currentWeather += i18n(" Ice Pellets");
+			phenomena = i18n("Ice Pellets");
 			weatherInfo.theWeather = "hail";
 		}
 		else if (s.contains("GR"))
 		{
-			currentWeather += i18n(" Hail");
+			phenomena = i18n("Hail");
 			weatherInfo.theWeather = "hail";
 		}
 		else if (sCode.contains("GS"))
 		{
-			currentWeather += i18n(" Small Hail Pellets");
+			phenomena = i18n("Small Hail Pellets");
 			weatherInfo.theWeather = "hail";
 		}
 		else if (s.contains("UP"))
 		{
-			currentWeather += i18n(" Unknown Precipitation");
+			phenomena = i18n("Unknown Precipitation");
 			weatherInfo.theWeather = iconName("shower1");
 		}
 		else if (sCode.contains("BR"))
 		{
-			currentWeather += i18n(" Mist");
+			phenomena = i18n("Mist");
 			// Mist has lower priority than say rain or snow
 			if ( weatherInfo.theWeather.isEmpty() )
 			{
@@ -309,7 +308,7 @@ bool MetarParser::parseCurrent(const QString &s)
 		}
 		else if (sCode.contains("FG"))
 		{
-			currentWeather += i18n(" Fog");
+			phenomena = i18n("Fog");
 			// Fog has lower priority than say rain or snow
 			if ( weatherInfo.theWeather.isEmpty() )
 			{
@@ -317,32 +316,34 @@ bool MetarParser::parseCurrent(const QString &s)
 			}
 		}
 		else if (sCode.contains("FU"))
-			currentWeather += i18n(" Smoke");
+			phenomena = i18n("Smoke");
 		else if (sCode.contains("VA"))
-			currentWeather += i18n(" Volcanic Ash");
+			phenomena = i18n("Volcanic Ash");
 		else if (sCode.contains("DU"))
-			currentWeather += i18n(" Widespread Dust");
+			phenomena = i18n("Widespread Dust");
 		else if (sCode.contains("SA"))
-			currentWeather += i18n(" Sand");
+			phenomena = i18n("Sand");
 		else if (sCode.contains("HZ"))
-			currentWeather += i18n(" Haze");
+			phenomena = i18n("Haze");
 		else if (sCode.contains("PY"))
-			currentWeather += i18n(" Spray");
+			phenomena = i18n("Spray");
 		else if (sCode.contains("PO"))
-			currentWeather += i18n(" Dust/Sand Swirls");
+			phenomena = i18n("Dust/Sand Swirls");
 		else if (sCode.contains("SQ"))
-			currentWeather += i18n(" Sudden Winds");
+			phenomena = i18n("Sudden Winds");
 		else if (sCode.contains("FC"))
 		{
 			if (sIntensity == "+")
 				currentWeather = i18n("Tornado");
 			else
-				currentWeather += i18n(" Funnel Cloud");
+				phenomena = i18n("Funnel Cloud");
 		}
 		else if (sCode.contains("SS"))
-			currentWeather += i18n(" Sand Storm");
+			phenomena = i18n("Sand Storm");
 		else if (sCode.contains("DS"))
-			currentWeather += i18n("Dust Storm");
+			phenomena = i18n("Dust Storm");
+		
+		if (currentWeather.isEmpty()) currentWeather = i18nc("%1 is the intensity, %2 is the descriptor and %3 is the phenomena", "%1 %2 %3", intensity, descriptor, phenomena);
 
 		if (!currentWeather.isEmpty())
 			weatherInfo.qsCurrentList << currentWeather;
@@ -357,7 +358,7 @@ bool MetarParser::parseTemperature(const QString &s)
 {
 	if (TempRegExp.search(s) > -1)
 	{
-		kdDebug(12006) << "Temp: " << TempRegExp.capturedTexts().join("-")
+		kDebug(12006) << "Temp: " << TempRegExp.capturedTexts().join("-")
 				<< endl;
 
 		float fTemp = TempRegExp.cap(2).toFloat();
@@ -378,7 +379,7 @@ bool MetarParser::parseTemperatureTenths(const QString &s)
 {
 	if (TempTenRegExp.search(s) > -1)
 	{
-		kdDebug(12006) << "Temp Tenths: " << TempTenRegExp.capturedTexts().join("-")
+		kDebug(12006) << "Temp Tenths: " << TempTenRegExp.capturedTexts().join("-")
 				<< endl;
 		
 		float temperature = TempTenRegExp.cap( 1 ).toFloat() / 10;
@@ -407,9 +408,11 @@ void MetarParser::calcTemperatureVariables()
 {
 #define E(t) ::pow(10, 7.5*t/(237.7+t))
 	float fRelHumidity = E(weatherInfo.dewC)/E(weatherInfo.tempC) * 100;
+        if (fRelHumidity > 100.0) fRelHumidity = 100.0; 
+
 	weatherInfo.qsRelHumidity.sprintf("%.1f", fRelHumidity);
 	removeTrailingDotZero(weatherInfo.qsRelHumidity);
-	weatherInfo.qsRelHumidity += "%";
+	weatherInfo.qsRelHumidity += '%';
 
 	float fHeatIndex = 0;
 	float tempF = TEMP_C_TO_F(weatherInfo.tempC);
@@ -470,7 +473,7 @@ bool MetarParser::parseDate(const QString &s)
 {
 	if (DateRegExp.search(s) > -1)
 	{
-		kdDebug(12006) << "Date: " << DateRegExp.capturedTexts().join("-")
+		kDebug(12006) << "Date: " << DateRegExp.capturedTexts().join("-")
 				<< endl;
 		QString dateString = DateRegExp.cap(1);
 		QString day, month, year;
@@ -493,7 +496,7 @@ bool MetarParser::parseTime(const QString &s)
 {
 	if (TimeRegExp.search(s) > -1)
 	{
-		kdDebug(12006) << "Time: " << TimeRegExp.capturedTexts().join("-")
+		kDebug(12006) << "Time: " << TimeRegExp.capturedTexts().join("-")
 				<< endl;
 
 		QString hour, minute, dateString;
@@ -518,7 +521,7 @@ bool MetarParser::parseVisibility(QStringList::ConstIterator it)
 	{
 		fVisibility = VisRegExp.cap(1).toFloat();
 
-		kdDebug(12006) << "Visibility: " << VisRegExp.capturedTexts().join("-")
+		kDebug(12006) << "Visibility: " << VisRegExp.capturedTexts().join("-")
 				<< endl;
 
 	}
@@ -561,7 +564,7 @@ bool MetarParser::parsePressure( const QString &s)
 		QString type = PressRegExp.cap(1);
 		float fPressure = PressRegExp.cap(2).toFloat();
 
-		kdDebug(12006) << "Pressure: " << PressRegExp.capturedTexts().join("-")
+		kDebug(12006) << "Pressure: " << PressRegExp.capturedTexts().join("-")
 				<< endl;
 
 		if (m_units == KLocale::Metric)
@@ -640,7 +643,7 @@ bool MetarParser::parseWindSpeed(const QString &s)
 		float gustSpeed = WindRegExp.cap(3).toFloat();
 		QString sWindUnit = WindRegExp.cap(4);
 
-		kdDebug(12006) << "Wind: " << WindRegExp.capturedTexts().join("-")
+		kDebug(12006) << "Wind: " << WindRegExp.capturedTexts().join("-")
 				<< endl;
 
 		if (m_units == KLocale::Metric)
@@ -656,7 +659,7 @@ bool MetarParser::parseWindSpeed(const QString &s)
 				gustSpeed = (gustSpeed * 3.6);
 			}
 			weatherInfo.windMPH = (windSpeed / 1.61);
-			weatherInfo.qsWindSpeed = i18n("1 km/h", "%n km/h", (int) windSpeed);
+			weatherInfo.qsWindSpeed = i18np("1 km/h", "%1 km/h", (int) windSpeed);
 		}
 		else
 		{
@@ -676,20 +679,20 @@ bool MetarParser::parseWindSpeed(const QString &s)
 				gustSpeed = (gustSpeed * 2.24);
 			}
 			weatherInfo.windMPH = windSpeed;
-			weatherInfo.qsWindSpeed = i18n("1 MPH", "%n MPH", (int) windSpeed);
+			weatherInfo.qsWindSpeed = i18np("1 MPH", "%1 MPH", (int) windSpeed);
 		}
 
 		if (gustSpeed >= 1)
 		{
 			if (m_units == KLocale::Metric)
 			{
-				weatherInfo.qsCurrentList << i18n("Wind gusts up to 1 km/h", 
-			     "Wind gusts up to %n km/h", (int) gustSpeed);
+				weatherInfo.qsCurrentList << i18np("Wind gusts up to 1 km/h", 
+			     "Wind gusts up to %1 km/h", (int) gustSpeed);
 			}
 			else
 			{
-				weatherInfo.qsCurrentList << i18n("Wind gusts up to 1 MPH", 
-			     "Wind gusts up to %n MPH", (int) gustSpeed);
+				weatherInfo.qsCurrentList << i18np("Wind gusts up to 1 MPH", 
+			     "Wind gusts up to %1 MPH", (int) gustSpeed);
 			}
 		}
 
@@ -707,7 +710,7 @@ bool MetarParser::parseStationNeedsMaintenance(const QString &s)
 	if (s == "$")
 	{
 		weatherInfo.stationNeedsMaintenance = true;
-		kdDebug(12006) << "Station Needs Maintenance" << endl;
+		kDebug(12006) << "Station Needs Maintenance";
 		return true;
 	}
 	
@@ -783,17 +786,17 @@ void MetarParser::calcCurrentIcon()
 			weatherInfo.theWeather = "cloudy5";
 	}
 
-	kdDebug(12006) << "Clouds: " << weatherInfo.clouds << ", Icon: "
+	kDebug(12006) << "Clouds: " << weatherInfo.clouds << ", Icon: "
 			<< weatherInfo.theWeather << endl;
 }
 
 void MetarParser::calcWindChill()
 {
 	float windChill = 35.74 + (0.6215 * TEMP_C_TO_F(weatherInfo.tempC))
-			- (35.75 * ::pow(weatherInfo.windMPH, 0.16))
-			+ (0.4275 * TEMP_C_TO_F(weatherInfo.tempC) * ::pow(weatherInfo.windMPH, 0.16));
+			- (35.75 * ::pow((double)weatherInfo.windMPH, (double)0.16))
+			+ (0.4275 * TEMP_C_TO_F(weatherInfo.tempC) * ::pow((double)weatherInfo.windMPH, (double)0.16));
 
-	kdDebug(12006) << "Wind Chill: " << windChill << endl;
+	kDebug(12006) << "Wind Chill: " << windChill;
 
 	if (windChill < 48)
 	{
@@ -814,7 +817,7 @@ void MetarParser::calcWindChill()
 
 bool MetarParser::isNight(const QString &stationID) const
 {
-	QString upperStationID = stationID.upper();
+	QString upperStationID = stationID.toUpper();
 	QString latitude  = m_stationDb->stationLatitudeFromID(upperStationID);
 	QString longitude = m_stationDb->stationLongitudeFromID(upperStationID);
 
@@ -832,12 +835,27 @@ bool MetarParser::isNight(const QString &stationID) const
 		QTime civilStart = theSun.computeCivilTwilightStart();
 		QTime civilEnd   = theSun.computeCivilTwilightEnd();
 
-		kdDebug (12006) << "station, current, lat, lon, start, end, offset: " << 
+		kDebug (12006) << "station, current, lat, lon, start, end, offset: " << 
 				upperStationID << " " << currently << " " << latitude << " " << 
 				longitude << " " << civilStart << " " << civilEnd << " " <<
 				m_localUTCOffset << endl;
 
-		return (currently < civilStart || currently > civilEnd);
+		if (civilStart != civilEnd)
+		{
+			if (civilEnd < civilStart)
+				/* Handle daylight past midnight in local time     */
+				/* for weather stations located at other timezones */
+				return (currently < civilStart && currently > civilEnd);
+			else
+				return (currently < civilStart || currently > civilEnd);
+		}
+		else
+		{
+			// Midnight Sun & Polar Night - In summer, the Sun is always 
+			// over the horizon line ... so use latitude & today date to 
+			// set isNight() value. 
+			return ((m_date.daysInYear() >= 80 || m_date.daysInYear() <= 264) && latitude.contains("S")); 
+		}
 	}
 }
 
